﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyProject.Domain;
using NHibernate;
using NHibernate.Cfg;
using MyProject.Services;
using MyProject.DomainPersistence;

namespace MyProject.ConsoleApp
{
    class Program
    {
        private static ISessionFactory _sessionFactory = BuildSessionFactory();

        private static ISessionFactory BuildSessionFactory()
        {
            var cfg = new Configuration();
            cfg.Configure();

            return cfg.BuildSessionFactory();
        }

        private static int CreateAndPersistCustomerWithOrders()
        {
            var order = new Order();

            order.AddItem(new OrderItem() { Cost = 1m, Quantity = 2, Description = "Soap" });
            order.AddItem(new OrderItem() { Cost = 1.5m, Quantity = 3, Description = "Book" });
            order.AddItem(new OrderItem() { Cost = 32.75m, Quantity = 1, Description = "iPod" });
            order.AddItem(new OrderItem() { Cost = 45.65m, Quantity = 2, Description = "WP7Phone" });

            var customers = new Customers(_sessionFactory.OpenSession());

            var customer = new Customer() { Firstname = "Steve", Lastname = "Bohlen" };

            customer.AddOrder(order);

            customers.Put(customer);

            return customer.Id;
        }

        private static void FixUpForShipping(Customer customer)
        {
            Console.WriteLine("\n\nFixing up the missing info in order to ship the order...");

            customer.PostalCode = "12345";

            var taxCalculator = new TaxCalculator(new TaxRateLookup());

            foreach (Order order in customer.Orders)
            {
                var tax = taxCalculator.Calculate(customer, order);

                order.ApplySalesTax(tax);
            }

        }

        private static Customer GetCustomerAndReportOnOrderItems(int customerId)
        {
            var customers = new Customers(_sessionFactory.OpenSession());

            var customer = customers.Get(customerId);

            foreach (Order retrievedOrder in customer.Orders)
            {
                Console.WriteLine("\n-------------\nOrder with Id: {0}\n", retrievedOrder.Id);

                foreach (OrderItem item in retrievedOrder.Items)
                {
                    Console.WriteLine("Item: {0}\t{1}\t{2} ({3})", item.Id, item.Quantity, item.Cost, item.Description);
                }

                Console.WriteLine("\n-------------\nTotal Cost: {0}", retrievedOrder.Cost);
            }

            return customer;
        }

        static void Main(string[] args)
        {
            //setup a customer with an order and items, persist it and return the ID for later
            var newCustomerId = CreateAndPersistCustomerWithOrders();

            //use the id to retreive the customer, orders, items from the DB
            var customer = GetCustomerAndReportOnOrderItems(newCustomerId);

            //attempt to ship all orders; will fail b/c the customer lacks a postal code
            //and order has no tax calculated yet
            ShipAllOrdersFor(customer);

            //add the missing postal code and calculate the missing tax
            FixUpForShipping(customer);

            //attempt to ship all orders for the customer again; should succeed this time
            ShipAllOrdersFor(customer);

            Console.WriteLine("\n\nHit any key to contine...");
            Console.ReadKey();
        }

        private static void ShipAllOrdersFor(Customer customer)
        {
            Console.WriteLine("\nAttempting to ship the order...");

            foreach (Order order in customer.Orders)
            {
                try
                {
                    ShipOrder(customer, order);
                }
                catch (CannotShipOrderException ex)
                {
                    Console.WriteLine("\nFailed to Ship order due to the following: {0}", ex.Message);
                }

            }
        }

        private static void ShipOrder(Customer customer, Order order)
        {
            var approver = new OrderApprover();
            var warehouse = new DomesticWarehouse();
            var dispatcher = new TruckDispatcher();
            var shipper = new OrderShipper(approver, warehouse, dispatcher);

            shipper.Ship(customer, order);

            Console.WriteLine("\nOrder Id: {0} has been shipped.", order.Id);
        }

    }
}
